/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "controllers/common/family/Families.h"
#include "DirectCamera.h"

using namespace std;

namespace dif_s {

const string DirectCamera::TYPE = "direct_camera";

/*
 * Constructor
 */
DirectCamera::DirectCamera(const string& name) :
		utilities::ImageData(name), controller::Stoppable(this) {

	setFamily(family::ACQUISITION, family::SETTING);

//	activated.init(this, NOSAVE, "activated");
	liveView.init(this, NOSAVE, "live_view");
	cameraPosition.init(this, SAVE, "camera_position");
	cameraReactor.init(this, SAVE, "camera_reactor");
	axisName.init(this, SAVE, "axis_name");
	startedPosition.init(this, SAVE, "started_position");
	endedPosition.init(this, SAVE, "ended_position");
	stepPosition.init(this, SAVE, "step_position");
	exposureTime.init(this, SAVE, "exposure_time");
	exposureTime.setUnit("s");
	rgbData.init(this, NOSAVE, "rgb_data");
	xSize.init(this, NOSAVE, "xsize");
	ySize.init(this, NOSAVE, "ysize");

	registerFunction(TYPE);

	m_Scan.init(this, "scan");
	m_Driver.init(this, "driver");

}

/*
 * Destructor
 */
DirectCamera::~DirectCamera() {
}

/*
 * postConfiguration
 */
void DirectCamera::postConfiguration() {
	xSize = m_Driver->width();
	ySize = m_Driver->height();
	registerUpdater(m_Driver->RGBData, &DirectCamera::updateRGBData, this);
}

/*
 * start
 */
void DirectCamera::start() {
	commandProgression = 0;

	try {

		if (liveView() == true) {
			commandStatus.setRunning();
			do {
				m_Driver.execute(driver::SYNCHRONIZE_READ_COMMAND, true);
			} while(isStopped() == false);
		}
		else {
			m_Scan->nProperties = 1;
			m_Scan->mode.set(0, scan::GenericScan::INTERVAL_PROPERTY);
			m_Scan->controller.set(0, axisName());
			m_Scan->wantedProperty.set(0, "wanted_position");
			m_Scan->offsetProperty.set(0, "");
			m_Scan->startValue.set(0, startedPosition());
			m_Scan->endValue.set(0, endedPosition());
			m_Scan->nSteps.set(0, stepPosition());
			m_Scan->countMode = "time";
			m_Scan->wantedTime = exposureTime();
			m_Scan->timeType = "s";

			log(Level::s_Info) << "Scan on " << axisName << " from " << startedPosition << " to " << endedPosition << " step "
					<< stepPosition << endlog;
			commandStatus.setRunning();
			m_Scan->startCommand(true);
			commandStatus.setIdle();
		}
	} catch (...) {
		commandStatus.setError();
	}
	commandProgression = 100;
}

/*
 * stop
 */
void DirectCamera::stop() {
	if (liveView() == false) {
		m_Scan->stopParallel(true);
	}
}

/*
 * updateProgression
 */
void DirectCamera::updateProgression() {
	if (isStarted() == true) {
		if (liveView() == false) {
			int32 progression = m_Scan->commandProgression();
			if (progression >= 100) {
				progression = 99;
			}
			commandProgression = progression;
		}
	}
}

void DirectCamera::updateRGBData() {
	rgbData = m_Driver->RGBData();
	type = m_Driver->pixelFormat();
	setContentFromBinaryData(rgbData());
}

}
