/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "DPPHistogram.h"
#include "InstrumentAbstraction/CommandNames.h"
#include "DataProvider/ChangeAspect.h"
#include <common/base/Convert.h>
#include <boost/thread.hpp>
#include <boost/bind.hpp>
#include <cstdlib>

namespace npp {

/*
 * Property names
 */
const std::string DPPHistogram::LIVEHISTO = "nomadhisto";
const char DPPHistogram::SHARED_MEMORY_NAME[] = "HistogramSharedMemory";
const char DPPHistogram::SHARED_MEMORY_DATA_NAME[] = "data";
const char DPPHistogram::SHARED_MEMORY_ADCRATES_NAME[] = "adc_rates";
const std::string DPPHistogram::HISTOGRAM = "histogram";

const std::string DPPHistogram::DATA_PATH = "/users/data/";

using namespace common;
using namespace boost;
using namespace boost::interprocess;

const std::string DPPHistogram::TYPE = "dpp_histogram";

DPPHistogram::DPPHistogram(const std::string& name) :
	DPPLiveProcess(name),
	_xData(0),
	_ADCRates(0),
	_segment(0),
	_liveCoincidenceThread(0),
	_liveRunning(false),
	_liveRefreshThread(0) {

	triggerChannel.init(this, SAVE, "trigger_channel");
	delay.init(this, SAVE, "delay");
	dT.init(this, SAVE, "dT");
	maxBlock.init(this, SAVE, "max_block");
	blockSize.init(this, SAVE, "block_size");
	test.init(this, SAVE, "test");
	testFileName.init(this, SAVE, "test_file_name");
	xData.init(this, NOSAVE, "x_data");
	one.init(this, NOSAVE, "one");
	refreshTimeS.init(this, SAVE, "refresh_time_s");

	histogram.init(this, NOSAVE, "histogram");
	ADCId.init(this, NOSAVE, "adc_id");
	ADCRate.init(this, NOSAVE, "adc_rate");
}

DPPHistogram::DPPHistogram(const DPPHistogram& controller) :
	DPPLiveProcess(controller),
	_xData(0),
	_ADCRates(0),
	_segment(0),
	_liveCoincidenceThread(0),
	_liveRunning(false),
	_liveRefreshThread(0) {

}

DPPHistogram::~DPPHistogram() {

	shared_memory_object::remove(SHARED_MEMORY_NAME);

	delete _segment;
}

void DPPHistogram::postConfiguration() {

	// ADC X data
	_xData = new int32[1];
	xData.set(_xData);
	xData.setSize(1);

	// ADC Y data
	shared_memory_object::remove(SHARED_MEMORY_NAME);
	_segment = new managed_shared_memory(create_only, SHARED_MEMORY_NAME, SEGMENT_SIZE);

	// allocating first
	_segment->construct<int32_t>(SHARED_MEMORY_DATA_NAME)[1](0);
	_ADCRates = _segment->construct<float64>(SHARED_MEMORY_ADCRATES_NAME)[128](0);

	histogram.resize(MAX_ADC);
	ADCId.resize(MAX_ADC);
	ADCRate.resize(MAX_ADC);

	// allocating
	for (int32 i = 0; i < MAX_ADC; i++) {
		ADCId.set(i, i);
	}

	resetData();
}

void DPPHistogram::start() {
	resetData();
	startLiveHisto();
}

void DPPHistogram::stop() {
	stopLiveHisto();
}

void DPPHistogram::raz() {
	resetData();
}

void DPPHistogram::resetData() {

	// number of elements
	int32_t size = numberOfChannels() * numberOfADCs();

	// ADC X data
	delete [] _xData;
	_xData = new int32[numberOfChannels()];

	// initializing
	for (int32 i = 0; i < numberOfChannels(); ++i) {
		_xData[i] = i;
	}

	xData.set(_xData);
	xData.setSize(numberOfChannels());

	// ADC Y data
	// destroying last array
	_segment->destroy<int32_t>(SHARED_MEMORY_DATA_NAME);

	// allocating new array
	int32_t * array = _segment->construct<int32_t>(SHARED_MEMORY_DATA_NAME)[size](0);

	// setting dynamic properties
	for (int32 i = 0; i < numberOfADCs(); i++) {
		histogram.set(i, (int32 *)&array[i * numberOfChannels()]);
		histogram.setSize(i, numberOfChannels());
	}

	for (int32 i = numberOfADCs(); i < MAX_ADC; i++) {
		histogram.set(i, 0);
		histogram.setSize(i, 0);
	}

	for (int32 i = 0; i < MAX_ADC; i++) {
		ADCRate.set(i, 0.0);
	}
}

void DPPHistogram::startLiveHisto() {

	sendProgressEvent(0);
	// if stop is taking time, waiting here
	cout << "waiting for synchronization before launching " << LIVEHISTO << endl;
	boost::mutex::scoped_lock lock(_mutex);

	_liveRunning = true;

	_liveCoincidenceThread = new thread(bind(&DPPHistogram::executeLiveHisto, this));
	_liveRefreshThread = new thread(bind(&DPPHistogram::executeRefreshHisto, this));
	sendProgressEvent(100);
}

void DPPHistogram::stopLiveHisto() {

	boost::mutex::scoped_lock lock(_mutex);

	_liveRunning = false;

	if (_liveCoincidenceThread != 0) {

		string command("kill -2 `pidof ");
		command += LIVEHISTO + "`";

		::system(command.c_str());

		_liveCoincidenceThread->join();
		delete _liveCoincidenceThread;
		_liveCoincidenceThread = 0;

		cout << "terminated " << LIVEHISTO << endl;
	}

	if (_liveRefreshThread != 0) {

		_liveRefreshThread->join();
		delete _liveRefreshThread;
		_liveRefreshThread = 0;
	}
}

void DPPHistogram::executeLiveHisto() {

	string command;

	command += LIVEHISTO + " ";

	if (!test()) {

		size_t size = fileName().size();

		if (size < 6) {
			string prefix(6 - size, '0');
			command += DATA_PATH + prefix + fileName()  + ".lst ";
		} else {
			command += DATA_PATH + fileName()  + ".lst ";
		}
	} else {
		command += testFileName() + " ";
	}

	command += to<string>(triggerChannel()) + " ";

	// set in 1e-8s
	command += to<string>(delay() / 10) + " ";
	command += to<string>(dT() / 10) + " ";

	command += "$HOME/.nomadserver/crateboardhistogram.txt ";

	command += to<string>(maxBlock()) + " ";
	command += to<string>(blockSize()) + " ";

	cout << command << endl;

	::system(command.c_str());
}

void DPPHistogram::executeRefreshHisto() {

	while (_liveRunning) {
		sleep(refreshTimeS());

		for (int32 i = 0; i < numberOfADCs(); ++i) {
			histogram.sendEvent(i);
		}

		// setting dynamic properties
		for (int32 i = 0; i < numberOfADCs(); i++) {
			ADCRate.set(i, _ADCRates[i]);
		}
	}
}

}
