/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealStereoElecDriver.h"
#include "StereoElecDriver.h"
#include "StereoElecDef.h"

#include <iostream>
#include <boost/thread.hpp>
#include <boost/bind.hpp>


using namespace std;

namespace stereo_elec {


const int32 RealStereoElecDriver::READ_COMMAND = 1;
const int32 RealStereoElecDriver::START_COMMAND = 2;
const int32 RealStereoElecDriver::STOP_COMMAND = 3;
const int32 RealStereoElecDriver::SHUTDOWN_COMMAND = 4;
const int32 RealStereoElecDriver::READINFO_COMMAND = 5;
const int32 RealStereoElecDriver::CONFIG_COMMAND = 6;
const int32 RealStereoElecDriver::READMTCA_COMMAND = 7;
/*
 * Constructor
 */
RealStereoElecDriver::RealStereoElecDriver(StereoElecDriver* owner) :
		StereoElecState(owner) {


    m_commandQueue = new common::Queue<int32>(1024);
    m_mtcaQueue = new common::Queue<int32>(1024);
    m_serverThread = new boost::thread(boost::bind(&RealStereoElecDriver::qtLoop, this));
    m_serverThread2 = new boost::thread(boost::bind(&RealStereoElecDriver::mtcaLoop, this));

}

/*
 * Destructor
 */
RealStereoElecDriver::~RealStereoElecDriver() {


	m_commandQueue->put(new int32(SHUTDOWN_COMMAND));
	m_serverThread->join();
	delete m_serverThread;
	delete m_commandQueue;
	m_serverThread2->join();
	delete m_serverThread2;
	delete m_mtcaQueue;

}

void RealStereoElecDriver::qtLoop() {

	int argc = 0;
	char **argv = 0;
	QCoreApplication qtApplication(argc, argv);

	TstereoDaqLib* daq=new TstereoDaqLib();

	while(true){
        int32 * command = m_commandQueue->get();
        if (*command == SHUTDOWN_COMMAND) {
        	delete daq;
        	break;
        } else if (*command == CONFIG_COMMAND) {
        	daq->createTrigEnMask(owner()->DAQParams);
        	cout << "createTrigEnMask ok" << endl;

        	daq->InitAMCProto(owner()->nfeboard());
        	daq->DoConfig(owner()->DAQParams);
        	cout << "DoConfig ok" << endl;
        	owner()->isconfigured=true;

		} else if (*command == START_COMMAND) {
			if(owner()->isconfigured()){
				if(!owner()->started){
					//owner()->startActivated=true;
					//for (int i=0;i<owner()->nslot();i++) cout << (int)owner()->DAQParams.isFEBoard_v.at(i);
					//cout << endl;
					daq->DoStart(owner()->DAQParams);
					cout << "DoStart ok" << endl;
					daq->preQtAcqLoop();
					m_TimeCounter.clear();
					owner()->nbevent=0;
					owner()->started=true;
					//owner()->startActivated =false;
				}
			}
			else{
				cout << "The  MTCA crate is not yet configured. Restart the server or configure it clicking the button DAQ Config" << endl;
			}

		}
		else if (*command == READ_COMMAND) {
			if(owner()->started and !owner()->m_StopActivated){
				try {
					daq->QtAcqLoop();
					owner()->nbevent=daq->getNbEvent();
				    }
				catch (const std::exception& ex)
				    {
				    cout << "std exception" << endl;
				    }
				catch (QString &str)
				    {
				    cout << "QString exception:" << str.toStdString() << endl;
				    }
			}
		}else if (*command == READINFO_COMMAND) {
			unsigned int accepted_cnt,rejected_cnt;
			if(owner()->started and !owner()->m_StopActivated){
				daq->QtAcqLoopCounter(accepted_cnt,rejected_cnt,false);
				cout << dec << "Trig accepted " << accepted_cnt << " rejected " << rejected_cnt << endl;
			}
		}else if (*command == STOP_COMMAND) {
			//owner()->startActivated =false;
			if(owner()->started){
				owner()->started=false;

				daq->DoStop();
				cout << "DoStop ok" << endl;
				daq->DoSave();
				cout << "DoSave ok" << endl;
			}
			//owner()->m_StopActivated = false;
		}

        delete command;
	}
}
/*
 * init
 */
void RealStereoElecDriver::init() {

	readConfig();

	int32 index = 0;
	owner()->DAQParams.triggerIPAdd = owner()->triggeripaddress();
	owner()->DAQParams.isNatR5=false;
	owner()->DAQParams.FEParams_v.resize(owner()->nfeboard());
	owner()->DAQParams.ledBoardInserted =false;
	for (int j = 0; j < owner()->nslot(); j++) {
		if (owner()->boardtype(j) == FEBOARD) {
			owner()->DAQParams.isFEBoard_v.at(j) = true;
			owner()->DAQParams.FEParams_v.at(index).FE8IPAdd=owner()->boardipaddress(j);
			owner()->DAQParams.FEParams_v.at(index).slot_num = j+1;
			index++;
		}else if (owner()->boardtype(j) == LEDBOARD) {
			owner()->DAQParams.isFEBoard_v.at(j) = false;
			owner()->DAQParams.ledBoardInserted =true;
			owner()->DAQParams.ledIPAdd=owner()->boardipaddress(j);
		}
		else{
			owner()->DAQParams.isFEBoard_v.at(j) = false;
		}
	}

}
/*
 * clear
 */
void RealStereoElecDriver::clear() {
}

/*
 * writeParam
 */
void RealStereoElecDriver::writeParam() {
}

/*
 * readInfos
 */
void RealStereoElecDriver::readInfos() {
	if(owner()->started and !owner()->m_StopActivated) m_commandQueue->put(new int32(READINFO_COMMAND),0);
}
/*
 * start
 */
void RealStereoElecDriver::start() {
	if(!owner()->started){
		cout << "Start command" << endl;
		m_commandQueue->put(new int32(START_COMMAND),1);
		owner()->startActivated=true;
		owner()->m_StopActivated = false;
	}
}

/*
 * stop
 */
void RealStereoElecDriver::stop() {
	if(owner()->started){
		owner()->startActivated=false;
		if(!owner()->m_StopActivated){
			cout << "Stop command" << endl;
			m_commandQueue->put(new int32(STOP_COMMAND),1);
		}
		owner()->m_StopActivated = true;
	}
}

/*
 * read
 */
void RealStereoElecDriver::read() {
	if(owner()->started and !owner()->m_StopActivated) m_commandQueue->put(new int32(READ_COMMAND),0);
}

/*
 * synchroniseRead
 */
void RealStereoElecDriver::synchroniseRead() {
	if(owner()->started) read();
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * readStatus
 */
void RealStereoElecDriver::readStatus() {
	//cout << "readStatus() : startActivated = " << owner()->startActivated() << " started " << owner()->started << endl;
	if (owner()->started == true) {
		bool finished = false;
		int32 mode = owner()->mode();

		float64 actual = m_TimeCounter.getTime();
		owner()->time.update(actual);
		if (mode == acquisition::GateCommon::TIME_MODE) {
			float64 target = owner()->time.setpoint();
		   // cout << "Actual time " << actual << " target time " << target << endl;
			if (actual >= target) {
				finished = true;
				actual = target;
			}

		} else if (mode == acquisition::GateCommon::COUNTER1_MODE) {
			int32 target = owner()->counts.setpoint();
			int32 actual = owner()->nbevent;
		//	cout << "Actual nb event " << actual << " target nb event " << target << endl;
			if (actual >= target) {
				finished = true;
			}
		} else if (mode == acquisition::GateCommon::COUNTER2_MODE) {
			int32 target = owner()->counts2.setpoint();
			int32 actual = owner()->nbevent;
		//	cout << "Actual nb event " << actual << " target nb event " << target << endl;
			if (actual >= target) {
				finished = true;
			}
		}

		if (finished) {
			//m_Status = COUNT_END;
			//owner()->status.update(m_Status);
			//owner()->computeGateStatus(m_Status);
			//owner()->computeAcquisitionStatus(0);
			owner()->computeDetectorStatus(0);
			//owner()->m_StopActivated = true;
			//owner()->startActivated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			stop();
		} else {
			//owner()->status.update(m_Status);
			//owner()->computeGateStatus(m_Status);
			//owner()->computeAcquisitionStatus(0);
			owner()->computeDetectorStatus(0);
			owner()->calculateProgression();
		}
	} else if(owner()->startActivated()){  // If a run is currently starting, nothing is done
		owner()->computeDetectorStatus(0);
		owner()->calculateProgression();
	} else{
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
		//m_Status = COUNT_END;
		//owner()->status.update(m_Status);
		//owner()->computeGateStatus(m_Status);
		//owner()->computeAcquisitionStatus(0);
		owner()->computeDetectorStatus(0);
	}
	//cout << "end readstatus() " << endl;
}

void RealStereoElecDriver::config() {
	m_commandQueue->put(new int32(CONFIG_COMMAND));
}

void RealStereoElecDriver::readConfig() {
	string board[NBSLOT];
	string ip[NBSLOT];
	string mac[NBSLOT];

	cout <<"Read config MTCA " << endl ;

	stereo_ipmi_lib.getCrateConfig(owner()->mchipaddress().c_str(),board,ip,mac,/*print*/true);
	for(int i=0;i<NBSLOT;i++){
		owner()->boardtype.set(i,board[i]);
		owner()->boardipaddress.set(i,ip[i]);
		owner()->boardmacaddress.set(i,mac[i]);
	}

}

void RealStereoElecDriver::readMonitoring() {
	//cout << "test mtca queue" << endl;
	//m_mtcaQueue->put(new int32(READMTCA_COMMAND));
}

void RealStereoElecDriver::mtcaLoop() {
	string board[NBSLOT];

	while(true){
	   int32 * command = m_mtcaQueue->get();
		for(int i=0;i<NBSLOT;i++){
			board[i]=owner()->boardtype(i).c_str();
			cout << board[i] << endl;
		}
		stereo_ipmi_lib.getMonitorings(owner()->mchipaddress().c_str(),board,monFE8,monLED,/*print*/false);
	}

}
}
